<?php

namespace Drupal\bigquery_integration\Service;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Google\Cloud\BigQuery\BigQueryClient;
use Google\Cloud\BigQuery\Dataset;
use Google\Cloud\BigQuery\Table;

/**
 * Helper service for BigQuery Integration module.
 */
class BigQueryIntegrationService {

  /**
   * The BigQuery client.
   *
   * @var Google\Cloud\BigQuery\BigQueryClient
   */
  protected $bigQuery;

  /**
   * Constructs a new BigQueryIntegrationService instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory interface.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory interface.
   */
  public function __construct(protected ConfigFactoryInterface $configFactory, protected LoggerChannelFactoryInterface $loggerFactory) {
    $config = $this->configFactory->get('bigquery_integration.settings');
    $credentialsPath = $config->get('json_path', FALSE);

    if (!$credentialsPath || !file_exists($credentialsPath)) {
      $this->loggerFactory->get('bigquery_integration')->error("The credentials file does not exists: @path", ['@path' => $credentialsPath]);
      return;
    }

    // Read and verify credentials.
    $credentials = json_decode(file_get_contents($credentialsPath), TRUE);
    if (!$credentials) {
      $this->loggerFactory->get('bigquery_integration')->error("The credentials file could not be read: @path", ['@path' => $credentialsPath]);
      return;
    }

    $this->bigQuery = new BigQueryClient(['keyFile' => $credentials]);
  }

  /**
   * Returns BigQuery client.
   *
   * @return Google\Cloud\BigQuery\BigQueryClient
   *   The BigQuery client.
   */
  public function getBigQueryClient(): ?BigQueryClient {
    if (!$this->bigQuery) {
      $this->loggerFactory->get('bigquery_integration')->warning("Could not retrieve the BigQuery client, the connection could not be established.");
      return NULL;
    }

    return $this->bigQuery;
  }

  /**
   * Executes a query with the BigClient client.
   *
   * @param string $query
   *   The query to execute.
   */
  public function executeQuery($query) {
    if (!$this->bigQuery) {
      $this->loggerFactory->get('bigquery_integration')->warning("The connection to BigQuery could not be established.");
      return [];
    }

    $job = $this->bigQuery->query($query);
    $results = $this->bigQuery->runQuery($job);

    return $results->rows();
  }

  /**
   * Get a dataset from BigQuery.
   *
   * @param string $dataset_name
   *   The dataset name.
   *
   * @return \Google\Cloud\BigQuery\Dataset|null
   *   A valid dataset or NULL.
   */
  public function getDataSet($dataset_name): ?Dataset {
    $dataset = $this->bigQuery->dataSet($dataset_name);
    if ($dataset->exists()) {
      return $dataset;
    }

    return NULL;
  }

  /**
   * Get a table from a BigQuery dataset.
   *
   * @param string $dataset_name
   *   The name of the dataset where the table is stored.
   * @param string $table_name
   *   The name of the table.
   *
   * @return \Google\Cloud\BigQuery\Table
   *   The table requested.
   */
  public function getTable(string $dataset_name, string $table_name): ?Table {
    $dataset = $this->getDataSet($dataset_name);
    if ($dataset) {
      $table = $dataset->table($table_name);
      if ($table->exists()) {
        return $table;
      }
    }

    return NULL;
  }

  /**
   * Insert a row into a table from a BigQuery dataset.
   *
   * @param string $dataset_name
   *   The name of the dataset where the table is stored.
   * @param string $table_name
   *   The name of the table where we want to insert the row.
   * @param array $row
   *   The row with the info as array ['field' => 'value'].
   *
   * @return bool
   *   TRUE or FALSE if the insertion was completed.
   */
  public function insertRow(string $dataset_name, string $table_name, array $row): bool {
    $table = $this->getTable($dataset_name, $table_name);
    if ($table->exists()) {
      // This method is from Google\Cloud\BigQuery\Table, not the service.
      $result = $table->insertRow($row);
      if ($result->isSuccessful()) {
        return TRUE;
      }

      // Log the error.
      $failed = $result->failedRows();
      $this->loggerFactory->get('bigquery_integration')->error('Error inserting row in BigQuery: @msg', [
        '@msg' => json_encode($failed),
      ]);
    }

    return FALSE;
  }

}
